type term = Term.term

let base = Config.share_dir ^ "/plugin_iprover"
  
let term_to_pattern ?(term_info="_") t =
  let h = Hashtbl.create 20 in
  let rec aux =
    function
    | Term.Fun(f, l, _) -> "Fun({ name = \"" ^ Symbol.to_string f ^ "\" },["
       ^ List.fold_left (fun s x -> s ^ aux x ^ ";") "" (Term.arg_to_list l)
       ^	"], " ^ term_info ^ ")"
    | Term.Var(var, _) ->
       try
	 let s = Hashtbl.find h var ^ "'" in
	 Hashtbl.add h var s; s
       with
	 Not_found ->
	   let s = Var.to_ocaml var in
		    Hashtbl.add h var s; s
  in
    let s = aux t in
    Hashtbl.fold (fun var varname s ->
      Hashtbl.remove h var;
      try
	s ^ " && equal " ^ varname ^ " " ^ Hashtbl.find h var
      with Not_found -> s)
      h (s ^ " when true")
      
let rec term_to_rhs = function
  | Term.Fun(f, l, fi) ->
     "create_fun_term (Plugin.find_symb " ^ string_of_int (Symbol.get_fast_key f)
     ^ ") ["
     ^ List.fold_left (fun s x -> s ^ term_to_rhs x ^ ";") "" (Term.arg_to_list l)
     ^	"]"
  | Term.Var(var, _) ->
     Var.to_ocaml var


let compiled = ref false
let temp_file = ref None

let add_rule t1 t2 =
  let temp =
    match !temp_file with
      None -> begin
	let temp = Filename.temp_file "plugin_iprover" ".ml" in
	match Sys.command ("cp " ^ base ^ ".ml.model " ^ temp) with
	  0 -> temp_file := Some(temp); temp
	| _ -> failwith ("could not copy " ^ base ^ ".ml.model") end
    | Some f -> f in
  let in_f = open_in temp in
  let out_f = open_out (temp ^ ".tmp") in
  try while true do
      let l = input_line in_f in
      if l = "(* add rules here *)" then
        begin (* print_endline "Adding rule"; *)
	output_string out_f
	  ("  | " ^ term_to_pattern t1 ^ " -> Some("
	   ^ term_to_rhs t2
	   ^ ")\n"); end;
      output_string out_f (l ^ "\n")
    done
  with End_of_file -> close_in in_f; close_out out_f;
    Sys.rename (temp ^ ".tmp") temp;
    compiled := false

let compile () =
  print_endline "Compiling...";
  let Some file = !temp_file in
  let command =
    if Dynlink.is_native then
      Config.ocamlopt ^ " -o " ^ file ^ ".cmxs -shared -I " ^ Config.share_dir ^ "  " ^ file
    else
      "ocamlc.opt -o " ^ file ^ ".cmo -c -I " ^ Config.share_dir ^ " " ^ file
  in
  match Sys.command command with
    0 -> begin try
	         Dynlink.loadfile
	           (file ^ if Dynlink.is_native then ".cmxs" else ".cmo");
	         compiled := true
      with
	Dynlink.Error e -> failwith (Dynlink.error_message e)
    end
  | _ -> failwith "compilation of the rewrite rules"

let normalize t =
  match !temp_file with
    None -> t
  | Some _ -> begin
    if not !compiled then compile ();
    let t' = !Plugin.normalize t in
    TermDB.add_ref t' Logic_interface.term_db_ref
  end

let remove_if_exists f =
  if Sys.file_exists f then Sys.remove f

let clean_up () =
  match !temp_file with
  | None -> ()
  | Some f ->
     Sys.remove f;
    let base = Filename.chop_extension f in
    remove_if_exists (base ^ ".cmi");
    remove_if_exists (base ^ ".cmx");
    remove_if_exists (base ^ ".o");
    remove_if_exists (f ^ ".cmxs");
    remove_if_exists (f ^ ".cmo")
